#include <QtCore/QDir>
#include <QtCore/QFile>
#include <QtGui/QCheckBox>

#include <kmessagebox.h>
#include <konq_operations.h>
#include <kurl.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <kimagefilepreview.h>
#include <kimageio.h>

#include "chfacedlg.h"
#include "settings.h"
#include "webcamui.h"
#include "webcam.h"

ChFaceDlg::ChFaceDlg(const QString &picsDir, QWidget *parent) : KDialog(parent, Qt::Dialog)
{
    setupUi(mainWidget());

    dir = picsDir;

    connect(lwImages, SIGNAL(clicked(const QModelIndex &)), this, SLOT(activateButtons()));

    connect(lwImages, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(accept()));
    connect(pbFromFile, SIGNAL(clicked()), this, SLOT(slotGetImageFromFile()));
    connect(pbDelete, SIGNAL(clicked()), this, SLOT(slotDelImage()));
    connect(pbFromCam, SIGNAL(clicked()), this, SLOT(slotShowCamDlg()));
    connect(this, SIGNAL(okClicked()), this, SLOT(accept()));

    WebCam numCam(50, this);
    QMap<int, QString> devices = numCam.getCamList();

    scanImages();
    resize(420, 500);
    enableButtonOk(false);
    pbDelete->setEnabled(false);
    pbFromCam->setEnabled(!devices.isEmpty());
}

void ChFaceDlg::scanImages()
{
    if (lwImages->count()) {
        lwImages->clear();
    }

    QDir imageDir(dir);

    if (!dir.isEmpty() && imageDir.exists()) {
            const QStringList imgList = imageDir.entryList(QDir::Files);
            for (QStringList::const_iterator i = imgList.constBegin(); i != imgList.constEnd(); ++i) {
                new QListWidgetItem(QIcon(dir + *i), (*i).section('.', 0, 0), lwImages);
            }
    }

    imageDir.setPath(KCFGUserAccount::userFaceDir());

    if (imageDir.exists()) {
        const QStringList imgList = imageDir.entryList(QDir::Files);

        for (QStringList::const_iterator i = imgList.constBegin(); i != imgList.constEnd(); ++i) {

                new QListWidgetItem(QIcon(KCFGUserAccount::userFaceDir() + *i),
                                    '/' + (*i) == KCFGUserAccount::customFaceFile() ?
                                    i18n("(Custom)") : (*i).section('.' ,0 ,0 ), lwImages );
        }
    }
}

QPixmap ChFaceDlg::getFaceImage() const
{
    if(lwImages->currentItem()){
        return lwImages->currentItem()->icon().pixmap(64);
    }
    return QPixmap();
}

void ChFaceDlg::activateButtons()
{
    enableButtonOk(true);
    pbDelete->setEnabled(true);
}

void ChFaceDlg::slotGetImageFromFile()
{
// I think that this feature is not necesary now, because I add a delete button
// Up to date all image are stored.
//     QCheckBox *chkSaveImg = new QCheckBox(i18n("&Save copy in custom faces folder for future use"), 0);

    KFileDialog openDlg(QDir::homePath(),
                        KImageIO::pattern(KImageIO::Reading), this);

    openDlg.setOperationMode(KFileDialog::Opening);
    openDlg.setCaption(i18n("Choose Image"));
    openDlg.setMode(KFile::File | KFile::LocalOnly);

    KImageFilePreview *imgPreview = new KImageFilePreview(&openDlg);
    openDlg.setPreviewWidget(imgPreview);
    if (openDlg.exec() == QDialog::Accepted){

        QImage pix(openDlg.selectedFile());
        if (pix.isNull()) {
            KMessageBox::sorry(this, i18n("There was an error loading the image."));
            return;
        } else {
            addCustomPixmap(pix, QFileInfo(openDlg.selectedFile()).fileName().section('.', 0, 0));
        }
    }
}

void ChFaceDlg::slotDelImage()
{
    QString path = KCFGUserAccount::userFaceDir() + lwImages->currentItem()->text();

    if (QFile::exists(path)) {

        int rsp = KMessageBox::warningYesNo(this, i18n("Do you really want to delete this image?"),
                                            i18n("Delete image"));
        if (rsp == KMessageBox::Yes) {
            KonqOperations::del(this, KonqOperations::TRASH, KUrl(path));

            delete lwImages->takeItem(lwImages->currentRow());
        }
    } else {
        KMessageBox::sorry(this,
                           i18n("You don't have appropriate permissions to delete this image"));
    }
}

void ChFaceDlg::slotShowCamDlg()
{
    WebCamUi *wui = new WebCamUi(this);
    if (wui->exec() == QDialog::Accepted && wui->isPhotoTaken()) {
        addCustomPixmap(wui->getImage(), wui->getImgName());
    }
    delete wui;
}

void ChFaceDlg::addCustomPixmap(QImage pix, QString name)
{
    if ((pix.width() > KCFGUserAccount::faceSize()) || (pix.height() > KCFGUserAccount::faceSize())) {
        // Should be no bigger than certain size.
        pix = pix.scaled(KCFGUserAccount::faceSize(),
                         KCFGUserAccount::faceSize(),
                         Qt::KeepAspectRatio);
    }

    QDir userFaces(KCFGUserAccount::userFaceDir());
    if (!userFaces.exists()) {
        userFaces.mkdir(userFaces.absolutePath());
    }

    pix.save(userFaces.absolutePath() + "/" + name , "PNG");

    QListWidgetItem *newFace = new QListWidgetItem(QIcon(QPixmap::fromImage(pix)),
                                                   name, lwImages);
    lwImages->scrollToItem(newFace);
    lwImages->setCurrentItem(newFace);
}
#include "chfacedlg.moc"
