#include <klocale.h>
#include <kmessagebox.h>

#include "webcam.h"

WebCam::WebCam(int interval, QObject *parent) : QObject(parent)
{
    intervalTime = interval;
    stopped = true;

    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(sendNewFrame()));
}

WebCam::~WebCam()
{
    if(!isStopped()){
        stop();
    }
}

void WebCam::setDevice(const int id)
{
    camId = id;
}

void WebCam::setInterval(const int interval)
{
    intervalTime = interval;

    if (!isStopped()) {
        timer->stop();
        timer->start(interval);
    }
}

bool WebCam::isStopped() const
{
    return stopped;
}

// QImage WebCam::getPreview()
// {
//     IplImage *frame = cvQueryFrame(cam);
//
//     return prepareFrame(frame);
// }

QMap<int, QString> WebCam::getCamList() const
{
    QMap<int, QString> camList;
    QList<Solid::Device> devices = Solid::Device::listFromType(Solid::DeviceInterface::Video,
                                                                QString());
    foreach (const Solid::Device &dev, devices) {
        QStringList protocols = dev.as<Solid::Video>()->supportedProtocols();

        if (protocols.contains("video4linux")) {
            QStringList drivers = dev.as<Solid::Video>()->supportedDrivers("video4linux");

            if (drivers.contains("video4linux")) {
                QString path = dev.as <Solid::Video>()->driverHandle("video4linux").toString();

                bool ok;
                int devNumber = path.mid(10).toInt(&ok, 10);
                if (ok){
                    camList.insert(devNumber,  dev.product());
                }
            }
        }
    }

    return camList;
}

void WebCam::stop()
{
    if (!isStopped()) {
        timer->stop();

        if (cam) {
            cvReleaseCapture(&cam);
        }
        stopped = true;
    }
}

void WebCam::start()
{
    if (isStopped()) {
        cam = cvCreateCameraCapture(camId);

        if (cam) {
            timer->start(intervalTime);
            stopped = false;
        } else {
            KMessageBox::sorry(0, i18n("Error while initializing the camera"));
        }
    }
}

QImage WebCam::prepareFrame(IplImage *frame)
{
    cvCvtColor(frame, frame, CV_BGR2RGB);
    cvFlip(frame, frame, 1);

    QImage image((uchar *)frame->imageData,
                  frame->width,
                  frame->height,
                  QImage::Format_RGB888);

//     preview = image;
    return image;
}

void WebCam::sendNewFrame()
{
    IplImage *frame = cvQueryFrame(cam);

    emit newFrame(prepareFrame(frame));
}
#include "webcam.moc"
