#include <QtGui/QInputDialog>
#include <kmessagebox.h>

#include "settings.h"
#include "webcamui.h"

WebCamUi::WebCamUi(QWidget *parent) : KDialog(parent, Qt::Dialog)
{
    photo = false;
    setupUi(mainWidget());
    cam = new WebCam(50, this);

    setCboCamera();
    connect(cboDevice, SIGNAL(activated(int)), this, SLOT(switchCamera(int)));
    connect(cam, SIGNAL(newFrame(QImage)), this, SLOT(showImage(QImage)));
    connect(this, SIGNAL(okClicked()), this, SLOT(saveImage()));

    switchCamera(cboDevice->currentIndex());
}

bool WebCamUi::isPhotoTaken() const
{
    return photo;
}

void WebCamUi::setCboCamera()
{
    QMap<int, QString> devices = cam->getCamList();
    QMap<int, QString>::const_iterator i;

    for (i = devices.constBegin(); i != devices.constEnd(); ++i) {
        cboDevice->addItem(i.value(), i.key());
    }
}

void WebCamUi::switchCamera(int index)
{
    if (!cam->isStopped()) {
        cam->stop();
    }
    cam->setDevice(cboDevice->itemData(index).toInt());
    cam->start();
}

void WebCamUi::showImage(QImage image)
{
    QSize size(lbVideo->width(), lbVideo->height());

    lbVideo->setPixmap(QPixmap::fromImage(image.scaled(size, Qt::IgnoreAspectRatio)));
}

QImage WebCamUi::getImage()
{
    return pix;
}

QString WebCamUi::getImgName() const
{
    return fname;
}

void WebCamUi::saveImage()
{

    cam->stop();
    fname = QInputDialog::getText(this, i18n("New Image"),
                                  i18n("Image name:"),
                                  QLineEdit::Normal,
                                  QString(), &photo);

    if (photo && !fname.isEmpty()) {
        pix = lbVideo->pixmap()->toImage();
    }
}


#include "webcamui.moc"
